// -*- coding: utf-8 -*-
// Skrypt używany przez options.xul
// Na wzór homepagescheduler-a

/////////////////////////////////////////////////////////////////
// Toolbar handling
/////////////////////////////////////////////////////////////////

function populateToolbarItems() {
    var prefString = smtext.prefman.getValue("toolbar", "");
    if( ! prefString ) {
        prefString = "/currentgames.aspx" + ".ITEM." + 'Game List'
            + ".NEXT."
            + "%PREVGAME%" + ".ITEM." + 'Prev Game'
            + ".NEXT."
            + "/forums.aspx" + ".ITEM." + 'Forum';
    }

    var toolbarData = prefString.split(".NEXT.");
    var toolbarItems = document.getElementById("toolbarItems");

    for (var i=0; i < toolbarData.length; i++) {
        var pieces = toolbarData[i].split(".ITEM.");

        var newItem = document.createElement("treeitem");
        var newRow = document.createElement("treerow");
        newItem.appendChild(newRow);

        var title = document.createElement("treecell");
        title.setAttribute("label", pieces[1]);
        newRow.appendChild(title);

        var url = document.createElement("treecell");
        url.setAttribute("label", pieces[0]);
        newRow.appendChild(url);

        toolbarItems.appendChild(newItem);
    }
}

function saveToolbarItems() {
    var toolbarItems = document.getElementById("toolbarItems").childNodes;
    var prefString = "";

    for (var i=0; i < toolbarItems.length; i++) {
        var columns = toolbarItems[i].childNodes[0].childNodes;
        var str = columns[1].getAttribute("label") + ".ITEM."
            + columns[0].getAttribute("label");
        if(prefString == "")
            prefString = str;
        else
            prefString += ".NEXT." + str;
    }

    /* return the new prefString to be stored by pref system */
    return prefString;
}


/*
 * Populates the edit part with the given info
 */
function populateToolbarEdit(titleStr, linkStr, editIndexStr) {
    var linkField = document.getElementById("linkField");
    var titleField = document.getElementById("titleField");
    var editIndex = document.getElementById("editIndex");

    editIndex.value = "" + editIndexStr;
    titleField.value = "" + titleStr;
    linkField.value = "" + linkStr;

    titleField.focus();
}

function editToolbarItem() {
    var toolbarItemsTree = document.getElementById("toolbarItemsTree");
    var selectedIndex = toolbarItemsTree.currentIndex;

    /* Ignore the button if nothing selected */
    if(selectedIndex == -1)
        return;

    var toolbarItems = document.getElementById("toolbarItems");
    var entry = toolbarItems.childNodes[selectedIndex].childNodes[0].childNodes;

    populateToolbarEdit(entry[0].getAttribute("label"),
                 entry[1].getAttribute("label"),
                 selectedIndex);

    flipToolbarView("edit");
}



/*
 * Deletes the currently selected item
 */
function deleteToolbarItem() {
    var toolbarItemsTree = document.getElementById("toolbarItemsTree");
    var index = toolbarItemsTree.currentIndex;

    if(index != -1) {
        var toolbarItems = document.getElementById("toolbarItems");
        var toRemove = toolbarItems.childNodes.item(index);
        toolbarItems.removeChild(toRemove);
        document.getElementById("paneTweaks").userChangedValue(toolbarItemsTree);
    }
}


/*
 * Flips the view from/to informational/edit modes
 */
function flipToolbarView(mode) {
    // var defaultGroupBox = document.getElementById("defaultGroupBox");
    var toolbarItemsGroupBox = document.getElementById("toolbarItemsGroupBox");
    var toolbarItemEditorGroupBox = document.getElementById("toolbarItemEditorGroupBox");

    if(mode == "edit") {
        // defaultGroupBox.setAttribute("hidden", "true");
        toolbarItemsGroupBox.setAttribute("hidden", "true");
        toolbarItemEditorGroupBox.setAttribute("hidden", "false");
    } else {
        // defaultGroupBox.setAttribute("hidden", "false");
        toolbarItemsGroupBox.setAttribute("hidden", "false");
        toolbarItemEditorGroupBox.setAttribute("hidden", "true");
    }
}



/*
 * Creates a new item
 */
function newToolbarItem() {
    flipToolbarView("edit");
    populateToolbarEdit("", "");
}



/*
 * Called when a new item has been created/modified
 */
function saveToolbarItem() {
    var linkField = document.getElementById("linkField");
    var titleField = document.getElementById("titleField");
    var editIndex = document.getElementById("editIndex");
    var toolbarItems = document.getElementById("toolbarItems");

    /* sanity checks */
    if(linkField.value == "") {
        alert("You must enter some link!");
        return;
    }
    if(titleField.value == "") {
        alert("You must enter some label!");
        return;
    }

    var newItem = document.createElement("treeitem");
    var newRow = document.createElement("treerow");
    newItem.appendChild(newRow);

    var title = document.createElement("treecell");
    title.setAttribute("label", titleField.value);
    newRow.appendChild(title);

    var url = document.createElement("treecell");
    url.setAttribute("label", linkField.value);
    newRow.appendChild(url);

    var oldIndex;
    if (editIndex.value) {
        oldIndex = parseInt(editIndex.value);
    }

    if (oldIndex) {
        var oldItem = toolbarItems.childNodes[oldIndex];
        toolbarItems.replaceChild(newItem, oldItem);
    } else {
        toolbarItems.appendChild(newItem);
    }

    var toolbarItemsTree = document.getElementById("toolbarItemsTree");
    document.getElementById("paneTweaks").userChangedValue(toolbarItemsTree);

    flipToolbarView("normal");
}

/*
 * Called when a new item that has been created/modified is cancelled.
 */
function cancelToolbarItem() {
    flipToolbarView("normal");
}

function moveToolbarItem(dir) {
    var toolbarItemsTree = document.getElementById("toolbarItemsTree");
    var index = toolbarItemsTree.currentIndex;

    if(index != -1) {
        var toolbarItems = document.getElementById("toolbarItems");
        if(dir == "up" && index > 0) {
            var nextIndex = index - 1;
            var top = toolbarItems.childNodes[nextIndex];
            var bottom = toolbarItems.childNodes[index];
        } else if(dir == "down" && index < toolbarItems.childNodes.length - 1) {
            var nextIndex = index + 1;
            var top = toolbarItems.childNodes[index];
            var bottom = toolbarItems.childNodes[nextIndex];
        } else {
            return;
        }

        var oA = top.childNodes[0].childNodes[0].getAttribute("label");
        var oB = top.childNodes[0].childNodes[1].getAttribute("label");
	
        var iA = bottom.childNodes[0].childNodes[0].getAttribute("label");
        var iB = bottom.childNodes[0].childNodes[1].getAttribute("label");
			
        top.childNodes[0].childNodes[0].setAttribute("label", iA);
        top.childNodes[0].childNodes[1].setAttribute("label", iB);

        bottom.childNodes[0].childNodes[0].setAttribute("label", oA);
        bottom.childNodes[0].childNodes[1].setAttribute("label", oB);

        toolbarItemsTree.currentIndex = nextIndex;
        toolbarItemsTree.focus();

        document.getElementById("paneTweaks").userChangedValue(toolbarItemsTree);
    }
}


function setToCurrentPage(element) {
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
        .getService(Components.interfaces.nsIWindowMediator);
    var win = wm.getMostRecentWindow("navigator:browser");
    if (win) {
        var homePageField = document.getElementById(element);
        var newVal = "";

        var tabbrowser = win.document.getElementById("content");
      	var l = tabbrowser.browsers.length;
        for (var i = 0; i < l; i++) {
            if (i)
                newVal += "|";
            newVal += tabbrowser.getBrowserAtIndex(i).webNavigation.currentURI.spec;
        }
      
      	homePageField.value = newVal;
    }
}

/////////////////////////////////////////////////////////////////
// Piece-set handling
/////////////////////////////////////////////////////////////////

// Dodaje do podanego obiektu DOM przyklad podanego
// piece-seta
function addSetSample(parent, setObj, size) {
    var pieces = [ 'hwkb', 'wbw', 'wnb', 'wrw', 'b', 
                   'hbkw', 'bbb', 'bnw', 'brb' ];
    for (var i = 0, piece; piece = pieces[i]; i++) {
        var newImage = document.createElement("image");
        newImage.setAttribute("src", setObj.pieceUrl(piece, size));
        parent.appendChild(newImage);
    }
}

// Dodaje do podanego obiektu DOM listę
// obrazków
function populatePieceRows(rows, size, selectedName) {
    var plist = smtpieces.allSuitable('chess', size);
    for(var pieceIdx = 0, pieceCount = plist.length;
        pieceIdx < pieceCount;
        ++ pieceIdx) {
        var setObj = plist[pieceIdx];
        var newRow = document.createElement("row");
        newRow.setAttribute("align", "center");
        newRow.appendChild(
           smtext.xulCreateButton(setObj.label(),
               "return saveSelectedSet('"+setObj.name()+"');"));
        addSetSample(newRow, setObj, size);
        rows.appendChild(newRow);
    }
}

function populatePieces(size, selectedName) {
    var oldList = document.getElementById("pieceSetSelectRows");
    if (oldList.firstChild && oldList.selectForSize == size) {
        return;
    }

    var newList = document.createElement("rows");
    populatePieceRows(newList, size, selectedName);
    newList.selectForSize = size;
    newList.setAttribute("id", "pieceSetSelectRows");

    oldList.parentNode.replaceChild(newList, oldList);
}

/*
 * Flips the view from/to informational/edit modes
 */
function flipPiecesView(mode, purpose) {
    var pieceSizeGroupBox = document.getElementById("pieceSizeGroupBox");
    var piecesByVariantGroupBox = document.getElementById("piecesByVariantGroupBox");
    var pieceSetSelectGroupBox = document.getElementById("pieceSetSelectGroupBox");

    if(mode == "edit") {
        pieceSizeGroupBox.setAttribute("hidden", "true");
        piecesByVariantGroupBox.setAttribute("hidden", "true");
        pieceSetSelectGroupBox.setAttribute("hidden", "false");
        pieceSetSelectGroupBox.purpose = purpose;
    } else {
        pieceSizeGroupBox.setAttribute("hidden", "false");
        piecesByVariantGroupBox.setAttribute("hidden", "false");
        pieceSetSelectGroupBox.setAttribute("hidden", "true");
        pieceSetSelectGroupBox.purpose = "";
    }
}

// Updates displayed name and sample images according to the current size
// and set. Returns setObj as it may be of some use
function updateDefaultSetInfo(specName, specSize) {
    //alert("xxx");
    var setName = specName
        || document.getElementById("pref_set_chess").value
        || smtext.prefman.getValue("set_chess");
    var setObj;
    if (setName) {
        setObj = smtpieces.pieceByName(setName);
    } else {
        setObj = smtpieces.firstMatchingPiece("chess");
        setName = setObj.name();
    }
    //alert(setName);
    var size = specSize
        || document.getElementById("pref_piece_size").value
        || smtext.prefman.getValue("piece_size")
        || '32';
    //alert(size);
    var oldBox = document.getElementById("chess_set_sample");

    //alert("oldBox: " + oldBox);
    document.getElementById("chess_set_label").value = setObj.label();
    if (oldBox.firstChild && oldBox.smt_set == setName && oldBox.smt_size == size) {
        return setObj;
    }

    var newBox = document.createElement("hbox");
    newBox.setAttribute("pack", "center");
    addSetSample(newBox, setObj, size);
    newBox.smt_set = setName;
    newBox.smt_size = size;
    newBox.setAttribute("id", "chess_set_sample");
    
    oldBox.parentNode.replaceChild(newBox, oldBox);

    // To generowało wyścigi
    //     removeDomChildren(oldBox);
    //     addSetSample(oldBox, setObj, size);
    //     oldBox.smt_set = setName;
    //     oldBox.smt_size = size;

    return setObj;
}

function onDefaultSetFromPref() {
    return updateDefaultSetInfo().name();
}

function onDefaultSetToPref() {
    // Chcemy zwrócić wartość defaultowego set-a...
    var x = document.getElementById("chess_set_name").value;
    //alert ("settopref: " + x);
    return x;
}

function onPiecePaneLoad() {
}

function onSizeChanged() {
    updateDefaultSetInfo();
}

function changeDefaultPieceSet() {
    var size = document.getElementById("pref_piece_size").value || 32;
    var set_chess = document.getElementById("pref_set_chess").value;
    populatePieces(size, set_chess);
    flipPiecesView('edit', 'default');
}

function cancelPieceSet() {
    flipPiecesView('normal');
}

function saveSelectedSet(setName) {
    var dsn = document.getElementById("chess_set_name");
    dsn.value=setName;
    document.getElementById("panePieceSets").userChangedValue(dsn);

    updateDefaultSetInfo();

    flipPiecesView('normal');
}

/////////////////////////////////////////////////////////////////
// Export/import
/////////////////////////////////////////////////////////////////

function exportSelFile(){
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
        .createInstance(nsIFilePicker);
    fp.init(window, "Select the file to save exported data", nsIFilePicker.modeSave);
    var filter = "*.smt";
    fp.appendFilter(filter,filter);

    var rv = fp.show();
    var file = fp.file;

    // file.path, file.leafName
    var ef = document.getElementById("exportFileId");
    ef.value = file.path;
    document.getElementById("paneExportImport").userChangedValue(ef);
    return true;
}

function importSelFile(){
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
        .createInstance(nsIFilePicker);
    fp.init(window, "Select the file to import data from", nsIFilePicker.modeOpen);
    var filter = "*.smt";
    fp.appendFilter(filter,filter);

    var rv = fp.show();
    var file = fp.file;

    // file.path, file.leafName
    document.getElementById("importFileId").value = file.path;
    return true;
}

var FIELDSEP = ":";
var LINESEP = "\n";

function getPrefsString() {
    var plist = smtext.prefman.all_prefs();
    var t = "";
    for(var i = 0, p; p = plist[i]; i++) {
        if (p == "import_file" || p == "export_file" || p == "data.PREVGAME") {
            continue;
        }
        var val = smtext.prefman.getValue(p, "");
        t = t + p + FIELDSEP + val + LINESEP;
    }
    return t;
}

function applyPreference(str) {
    var parts = str.split(FIELDSEP, 2);
    if (parts.length == 2) {
        var name = parts[0];
        var value = parts[1];
        if (value == "true") {
            value = true;
        }
        else if (value == "false") {
            value = false;
        }
        else if (value.match(/^\d+$/)) {
            value = parseInt(value);
        }
        smtext.dump("setting " + name + " = " + value);
        smtext.prefman.setValue(name, value);
    }
    else {
        smtext.dump("problems interpretting pref: " + str);
    }
}

function exportSettings() {
    var path = document.getElementById("exportFileId").value;
    if (! path) {
        alert("You must select some export file!");
        return false;
    }

    var exportFile = Components.classes["@mozilla.org/file/local;1"]
        .createInstance(Components.interfaces.nsILocalFile);
    exportFile.initWithPath(path);
    // exportFile.append(name);

    var exportStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
        .createInstance(Components.interfaces.nsIFileOutputStream);
    exportStream.init(exportFile, 0x02 | 0x08 | 0x20, 0755, 0); // write, create, truncate

    var charset = "UTF-8";
    var os = Components.classes["@mozilla.org/intl/converter-output-stream;1"]
        .createInstance(Components.interfaces.nsIConverterOutputStream);

    os.init(exportStream, charset, 4096, 0x0000);
    os.writeString(getPrefsString());
    os.close();

    alert("Settings succesfully exported");

    return true;
}

function importSettings() {
    var path = document.getElementById("importFileId").value;
    if (! path) {
        alert("You must select some import file!");
        return false;
    }

    var importFile = Components.classes["@mozilla.org/file/local;1"]
        .createInstance(Components.interfaces.nsILocalFile);
    importFile.initWithPath(path);
    // importFile.append(name);

    var importStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
        .createInstance(Components.interfaces.nsIFileInputStream);
    importStream.init(importFile, 0x01, 0444, 0);
    importStream.QueryInterface(Components.interfaces.nsILineInputStream);

    do {
        var line = {};
        var hasmore = importStream.readLine(line);
        applyPreference(line.value);
    } while (hasmore);

    importStream.close();

    alert("Settings succesfully imported");

    return true;
}